"""Added SecurityIndicatorAggregatedRun Table and Aggregated Table

Revision ID: 9aa4e29fa260
Revises: d77154f3c15f
Create Date: 2020-07-03 12:23:50.615984+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '9aa4e29fa260'
down_revision = 'd77154f3c15f'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('si_aggregated_run',
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('si_aggregated_run_document_id', sa.Text(), nullable=False),
    sa.Column('severity_high_confidence_high', sa.Integer(), nullable=False),
    sa.Column('severity_high_confidence_low', sa.Integer(), nullable=False),
    sa.Column('severity_high_confidence_medium', sa.Integer(), nullable=False),
    sa.Column('severity_high_confidence_undefined', sa.Integer(), nullable=False),
    sa.Column('severity_low_confidence_high', sa.Integer(), nullable=False),
    sa.Column('severity_low_confidence_low', sa.Integer(), nullable=False),
    sa.Column('severity_low_confidence_medium', sa.Integer(), nullable=False),
    sa.Column('severity_low_confidence_undefined', sa.Integer(), nullable=False),
    sa.Column('severity_medium_confidence_high', sa.Integer(), nullable=False),
    sa.Column('severity_medium_confidence_low', sa.Integer(), nullable=False),
    sa.Column('severity_medium_confidence_medium', sa.Integer(), nullable=False),
    sa.Column('severity_medium_confidence_undefined', sa.Integer(), nullable=False),
    sa.Column('number_of_analyzed_files', sa.Integer(), nullable=False),
    sa.Column('number_of_files_total', sa.Integer(), nullable=False),
    sa.Column('number_of_files_with_severities', sa.Integer(), nullable=False),
    sa.Column('number_of_filtered_files', sa.Integer(), nullable=False),
    sa.Column('number_of_python_files', sa.Integer(), nullable=False),
    sa.Column('number_of_lines_with_comments_in_python_files', sa.Integer(), nullable=False),
    sa.Column('number_of_blank_lines_in_python_files', sa.Integer(), nullable=False),
    sa.Column('number_of_lines_with_code_in_python_files', sa.Integer(), nullable=False),
    sa.Column('total_number_of_files', sa.Integer(), nullable=False),
    sa.Column('total_number_of_lines', sa.Integer(), nullable=False),
    sa.Column('total_number_of_lines_with_comments', sa.Integer(), nullable=False),
    sa.Column('total_number_of_blank_lines', sa.Integer(), nullable=False),
    sa.Column('total_number_of_lines_with_code', sa.Integer(), nullable=False),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('si_aggregated',
    sa.Column('si_aggregated_run_id', sa.Integer(), nullable=False),
    sa.Column('python_package_version_entity_id', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['python_package_version_entity_id'], ['python_package_version_entity.id'], ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['si_aggregated_run_id'], ['si_aggregated_run.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('si_aggregated_run_id', 'python_package_version_entity_id')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('si_aggregated')
    op.drop_table('si_aggregated_run')
    # ### end Alembic commands ###



"""Create index for solved on version id

Revision ID: 7d970baad7f2
Revises: b91c11e75551
Create Date: 2020-01-13 22:40:23.898138+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '7d970baad7f2'
down_revision = 'b91c11e75551'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index('solved_version_id_idx', 'solved', ['version_id'], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index('solved_version_id_idx', table_name='solved')
    # ### end Alembic commands ###



"""Allow nullable references for software environments

Revision ID: 54d50b0eeb33
Revises: e05d6d95ea95
Create Date: 2019-10-21 11:54:17.054303+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '54d50b0eeb33'
down_revision = 'e05d6d95ea95'
branch_labels = None
depends_on = None


def upgrade():
    op.drop_constraint('has_symbol_pkey', 'has_symbol', type_='primary')
    op.create_primary_key('has_symbol_pkey', 'has_symbol', ['id', ])
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('has_symbol', 'external_software_environment_id',
               existing_type=sa.INTEGER(),
               nullable=True)
    op.alter_column('has_symbol', 'software_environment_id',
               existing_type=sa.INTEGER(),
               nullable=True)
    op.alter_column('has_symbol', 'versioned_symbol_id',
               existing_type=sa.INTEGER(),
               nullable=True)
    # ### end Alembic commands ###


def downgrade():
    op.drop_constraint('has_symbol_pkey', 'has_symbol', type_='primary')
    op.create_primary_key('has_symbol_pkey', 'has_symbol', ['id', 'software_environment_id'])
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('has_symbol', 'versioned_symbol_id',
               existing_type=sa.INTEGER(),
               nullable=False)
    op.alter_column('has_symbol', 'software_environment_id',
               existing_type=sa.INTEGER(),
               nullable=False)
    op.alter_column('has_symbol', 'external_software_environment_id',
               existing_type=sa.INTEGER(),
               nullable=False)
    # ### end Alembic commands ###



#!/usr/bin/env python3
# thoth-storages
# Copyright(C) 2018, 2019, 2020 Fridolin Pokorny
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Adapter for persisting Amun inspection results."""

from .result_base import ResultStorageBase


class AnalysisByDigest(ResultStorageBase):
    """Adapter for storing image analysis results by its digest."""

    RESULT_TYPE = "analysis-by-digest"
    SCHEMA = None

    def store_document(self, image_digest: str, analysis_info: str) -> dict:
        """Store the given document representing analysis info onto Ceph."""
        document_id = self.ceph.store_document(analysis_info, document_id=image_digest)
        return document_id



#!/usr/bin/env python3
# thoth-storages
# Copyright(C) 2018, 2019, 2020 Fridolin Pokorny
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Adapter for persisting Amun inspection results."""

import os
import logging
from typing import Any
from typing import Dict
from typing import Generator
from typing import Optional

from .ceph import CephStore

_LOGGER = logging.getLogger(__name__)


def _get_inspection_prefix(inspection_id: Optional[str] = None) -> str:
    """Get prefix where inspections store data.

    This configuration matches Amun configmap.
    """
    bucket_prefix = os.environ["THOTH_CEPH_BUCKET_PREFIX"]
    deployment_name = os.environ["THOTH_DEPLOYMENT_NAME"]

    if inspection_id is None:
        return f'{bucket_prefix}/{deployment_name}/inspections'

    return f'{bucket_prefix}/{deployment_name}/inspections/{inspection_id}'


class _InspectionBase:
    """A base class for inspection builds and results."""

    __slots__ = ["ceph", "inspection_id"]

    def connect(self) -> None:
        """Connect this adapter to Ceph."""
        self.ceph.connect()

    def is_connected(self) -> bool:
        """Check if this adapter is connected."""
        return self.ceph.is_connected()

    def check_connection(self) -> None:
        """Check connection of this adapter."""
        return self.ceph.check_connection()


class InspectionBuildsStore(_InspectionBase):
    """An adapter for manipulating with inspection builds."""

    def __init__(self, inspection_id: str) -> None:
        """Constructor."""
        prefix = f"{_get_inspection_prefix(inspection_id)}/build/"
        self.ceph = CephStore(prefix=prefix)
        self.inspection_id = inspection_id

    def retrieve_dockerfile(self) -> str:
        """Retrieve Dockerfile used during the build."""
        return self.ceph.retrieve_blob("Dockerfile").decode()

    def retrieve_log(self) -> str:
        """Retrieve logs (stdout together with stderr) reported during the build."""
        return self.ceph.retrieve_blob("log").decode()

    def retrieve_specification(self) -> Dict[str, Any]:
        """Retrieve specification used for the build, captures also run specification."""
        return self.ceph.retrieve_document("specification")


class InspectionResultsStore(_InspectionBase):
    """An adapter for manipulating with inspection results."""

    def __init__(self, inspection_id: str) -> None:
        """Constructor."""
        prefix = f"{_get_inspection_prefix(inspection_id)}/results/"
        self.ceph = CephStore(prefix=prefix)
        self.ceph.connect()
        self.inspection_id = inspection_id

    @classmethod
    def get_document_id(cls, document: Dict[str, Any]) -> str:
        """Get id under which the given document will be stored."""
        return document["inspection_id"]

    def get_results_count(self) -> int:
        """Obtain number of results produced during inspection run."""
        items = []
        items_set = set()
        for object_key in self.ceph.get_document_listing():
            item, _ = object_key.split("/", maxsplit=1)
            item_int = int(item)
            if item_int not in items_set:
                items_set.add(item_int)
                items.append(item_int)

        del items_set

        if len(items) == 0:
            return 0

        items.sort(reverse=False)

        if len(items) != items[-1] + 1:
            _LOGGER.warning("Some of the inspection results are missing")

        return items[-1] + 1

    def retrieve_hwinfo(self, item: int) -> Dict[str, Any]:
        """Obtain hardware information for the given inspection run."""
        return self.ceph.retrieve_document(f"{item}/hwinfo")

    def retrieve_log(self, item: int) -> str:
        """Obtain log for the given inspection run."""
        return self.ceph.retrieve_blob(f"{item}/log").decode()

    def retrieve_result(self, item: int) -> Dict[str, Any]:
        """Obtain the actual result for the given inspection run."""
        return self.ceph.retrieve_document(f"{item}/result")

    def iter_inspection_results(self) -> Generator[Dict[str, Any], None, None]:
        """Iterate over inspection results."""
        for item in range(self.get_results_count()):
            yield self.retrieve_result(item)


class InspectionStore:
    """Adapter for manipulating with Amun inspections."""

    __slots__ = ["build", "results", "inspection_id"]

    def __init__(self, inspection_id: str) -> None:
        """A representation of an inspection."""
        self.inspection_id = inspection_id
        self.build = InspectionBuildsStore(inspection_id)
        self.results = InspectionResultsStore(inspection_id)

    def retrieve_specification(self) -> Dict[str, Any]:
        """Retrieve specification used for this inspection."""
        return self.build.retrieve_specification()

    def connect(self) -> None:
        """Connect this adapter."""
        self.build.connect()
        self.results.connect()

    def is_connected(self) -> bool:
        """Check if this adapter is connected."""
        return self.build.is_connected() and self.results.is_connected()

    def check_connection(self):
        """Check connections of this adapter."""
        self.build.check_connection()
        self.results.check_connection()

    def exists(self) -> bool:
        """Check if the given inspection exists."""
        # Specification is stored as one of the very first inspection results.
        return self.build.ceph.document_exists("specification")

    @classmethod
    def iter_inspections(cls) -> Generator[str, None, None]:
        """Iterate over inspection ids stored."""
        ceph = CephStore(prefix=_get_inspection_prefix())
        ceph.connect()

        last_id = None
        for item in ceph.get_document_listing():
            inspection_id = item.split("/", maxsplit=1)[0]
            if last_id == inspection_id:
                # Return only unique inspection ids, discard any results placed under the given prefix.
                continue

            last_id = inspection_id
            yield inspection_id

    @classmethod
    def get_inspection_count(cls) -> int:
        """Get number of inspection stored."""
        return sum(1 for _ in cls.iter_inspections())



"""Relax constraints for ExternalSoftwareEnvironment and SoftwareEnvironment for PackageExtractRun

Revision ID: 331cbc965699
Revises: 079e1d92f9d4
Create Date: 2019-10-08 15:51:15.249467+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '331cbc965699'
down_revision = '079e1d92f9d4'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('package_extract_run', 'external_software_environment_id',
               existing_type=sa.INTEGER(),
               nullable=True)
    op.alter_column('package_extract_run', 'software_environment_id',
               existing_type=sa.INTEGER(),
               nullable=True)
    op.alter_column('python_package_version', 'package_version',
               existing_type=sa.VARCHAR(length=256),
               nullable=True)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('python_package_version', 'package_version',
               existing_type=sa.VARCHAR(length=256),
               nullable=False)
    op.alter_column('package_extract_run', 'software_environment_id',
               existing_type=sa.INTEGER(),
               nullable=False)
    op.alter_column('package_extract_run', 'external_software_environment_id',
               existing_type=sa.INTEGER(),
               nullable=False)
    # ### end Alembic commands ###



"""Rename si_downloadable flag to is_si_analyzable

Revision ID: cd34fd8a5a90
Revises: c8017a5b43f1
Create Date: 2020-09-29 07:58:34.949144+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'cd34fd8a5a90'
down_revision = 'c8017a5b43f1'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('python_package_version', sa.Column('is_si_analyzable', sa.Boolean(), nullable=False, server_default="true"))
    op.drop_column('python_package_version', 'is_downloadable')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('python_package_version', sa.Column('is_downloadable', sa.BOOLEAN(), server_default=sa.text('true'), autoincrement=False, nullable=False))
    op.drop_column('python_package_version', 'is_si_analyzable')
    # ### end Alembic commands ###



"""Add indexes for abi_compatability queries

Revision ID: 0c8eab5d4197
Revises: bb26306538f0
Create Date: 2020-02-05 10:54:52.119190+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '0c8eab5d4197'
down_revision = 'bb26306538f0'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('build_log_analyzer_run',
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('build_log_analyzer_name', sa.Text(), nullable=True),
    sa.Column('build_log_analyzer_version', sa.Text(), nullable=True),
    sa.Column('build_log_analysis_document_id', sa.Text(), nullable=False),
    sa.Column('datetime', sa.DateTime(), nullable=False),
    sa.Column('debug', sa.Boolean(), nullable=False),
    sa.Column('build_log_analyzer_error_reason', sa.Text(), nullable=True),
    sa.Column('duration', sa.Integer(), nullable=True),
    sa.Column('input_python_package_version_entity_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['input_python_package_version_entity_id'], ['python_package_version_entity.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_index('python_package_version_entity_id_idx', 'python_package_version_entity', ['id'], unique=True)
    op.create_index('requires_symbol_python_artifact_id_idx', 'requires_symbol', ['python_artifact_id'], unique=False)
    op.create_index('versioned_symbol_id_idx', 'versioned_symbol', ['id'], unique=True)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index('versioned_symbol_id_idx', table_name='versioned_symbol')
    op.drop_index('requires_symbol_python_artifact_id_idx', table_name='requires_symbol')
    op.drop_index('python_package_version_entity_id_idx', table_name='python_package_version_entity')
    op.drop_table('build_log_analyzer_run')
    # ### end Alembic commands ###



"""Remove package-analyzer

Revision ID: 6389973bd8b6
Revises: 84588707e6ba
Create Date: 2020-09-14 08:31:09.089734+00:00

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = '6389973bd8b6'
down_revision = '84588707e6ba'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('investigated')
    op.drop_table('investigated_file')
    op.drop_table('package_analyzer_run')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('investigated_file',
    sa.Column('package_analyzer_run_id', sa.INTEGER(), autoincrement=False, nullable=False),
    sa.Column('python_file_digest_id', sa.INTEGER(), autoincrement=False, nullable=False),
    sa.ForeignKeyConstraint(['package_analyzer_run_id'], ['package_analyzer_run.id'], name='investigated_file_package_analyzer_run_id_fkey', ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['python_file_digest_id'], ['python_file_digest.id'], name='investigated_file_python_file_digest_id_fkey', ondelete='CASCADE')
    )
    op.create_table('package_analyzer_run',
    sa.Column('id', sa.INTEGER(), server_default=sa.text("nextval('package_analyzer_run_id_seq'::regclass)"), autoincrement=True, nullable=False),
    sa.Column('package_analyzer_name', sa.TEXT(), autoincrement=False, nullable=True),
    sa.Column('package_analyzer_version', sa.TEXT(), autoincrement=False, nullable=True),
    sa.Column('package_analysis_document_id', sa.TEXT(), autoincrement=False, nullable=False),
    sa.Column('datetime', postgresql.TIMESTAMP(), autoincrement=False, nullable=False),
    sa.Column('debug', sa.BOOLEAN(), autoincrement=False, nullable=False),
    sa.Column('package_analyzer_error', sa.BOOLEAN(), autoincrement=False, nullable=False),
    sa.Column('duration', sa.INTEGER(), autoincrement=False, nullable=True),
    sa.Column('input_python_package_version_entity_id', sa.INTEGER(), autoincrement=False, nullable=True),
    sa.ForeignKeyConstraint(['input_python_package_version_entity_id'], ['python_package_version_entity.id'], name='package_analyzer_run_input_python_package_version_entity_i_fkey', ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id', name='package_analyzer_run_pkey'),
    postgresql_ignore_search_path=False
    )
    op.create_table('investigated',
    sa.Column('package_analyzer_run_id', sa.INTEGER(), autoincrement=False, nullable=False),
    sa.Column('python_artifact_id', sa.INTEGER(), autoincrement=False, nullable=False),
    sa.ForeignKeyConstraint(['package_analyzer_run_id'], ['package_analyzer_run.id'], name='investigated_package_analyzer_run_id_fkey', ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['python_artifact_id'], ['python_artifact.id'], name='investigated_python_artifact_id_fkey', ondelete='CASCADE')
    )
    # ### end Alembic commands ###



#!/usr/bin/env python3
# thoth-storages
# Copyright(C) 2019, 2020 Francesco Murdaca
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Utilities for SQLAlchemy query results."""

import logging

from typing import List, Dict, Any, Optional, Union, Tuple

_LOGGER = logging.getLogger(__name__)


class PythonQueryResult:
    """Class for managing queries' results for Python Packages."""

    def __init__(
        self,
        result: Union[List, Dict[str, Any]],
        count: Optional[int] = None
    ):
        """Query result initialization."""
        self._result = result
        self._count = count

    @property
    def result(self):
        """Query result."""
        return self._result

    @property
    def count(self):
        """Query count result."""
        return self._count



"""Add result number to inspectionRun

Revision ID: d6d0b20ec650
Revises: 10925df620b1
Create Date: 2020-07-08 16:07:37.994149+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'd6d0b20ec650'
down_revision = '10925df620b1'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('inspection_run', sa.Column('inspection_result_number', sa.Integer(), nullable=False, server_default="0"))
    op.drop_constraint('inspection_run_inspection_document_id_key', 'inspection_run', type_='unique')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint('inspection_run_inspection_document_id_key', 'inspection_run', ['inspection_document_id'])
    op.drop_column('inspection_run', 'inspection_result_number')
    # ### end Alembic commands ###
